<?php
/**
 * Plugin Name: Appender
 * Plugin URI: https://1.envato.market/appender
 * Description: Adds copyrights to the copied text.
 * Author: پرشین اسکریپت
 * Author URI: www.persianscript.ir
 * Version: 1.0.1
 * Requires PHP: 5.6
 * Requires at least: 3.0
 * Tested up to: 5.3.2
 **/

/**
 * Appender - Adds copyrights to the copied text.
 * Exclusively on Envato Market: https://1.envato.market/appender
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       Copyright (C) 2018 - 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         dmitry@merkulov.design
 **/

namespace Merkulove;

use Merkulove\Appender\AssignmentsTab;
use Merkulove\Appender\PluginUpdater;
use Merkulove\Appender\Helper;
use Merkulove\Appender\PluginHelper;
use Merkulove\Appender\Settings;
use Merkulove\Appender\EnvatoItem;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';

/**
 * SINGLETON: Core class used to implement a Appender plugin.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @since 1.0.0
 */
final class Appender {

	/**
	 * Plugin version.
	 *
	 * @string version
	 * @since 1.0.0
	 **/
	public static $version = '';

	/**
	 * Use minified libraries if SCRIPT_DEBUG is turned off.
	 *
	 * @since 1.0.0
	 **/
	public static $suffix = '';

	/**
	 * URL (with trailing slash) to plugin folder.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $url = '';

	/**
	 * PATH to plugin folder.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $path = '';

	/**
	 * Plugin base name.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $basename = '';

	/**
	 * The one true Appender.
	 *
	 * @var Appender
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new plugin instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Initialize main variables. */
		$this->initialization();

		/** Define admin hooks. */
		$this->admin_hooks();

		/** Define public hooks. */
		$this->public_hooks();

		/** Define hooks that runs on both the front-end as well as the dashboard. */
		$this->both_hooks();

	}

	/**
	 * Return plugin version.
	 *
	 * @return string
	 * @since 1.0.0
	 * @access public
	 **/
	public function get_version() {
		return self::$version;
	}

	/**
	 * Define hooks that runs on both the front-end as well as the dashboard.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	private function both_hooks() {

		/** Load translation. */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );

	}

	/**
	 * Register all of the hooks related to the admin area functionality.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	private function admin_hooks() {

		/** Add plugin settings page. */
		Settings::get_instance()->add_settings_page();

		/** Load JS and CSS for Backend Area. */
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ], 100 ); // CSS.
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ], 100 ); // JS.

		/** Remove "Thank you for creating with WordPress" and WP version only from plugin settings page. */
		add_action( 'admin_enqueue_scripts', [$this, 'remove_wp_copyrights'] );

		/** Remove all "third-party" notices from plugin settings page. */
		add_action( 'in_admin_header', [$this, 'remove_all_notices'], 1000 );

	}

	/**
	 * Remove all other notices.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function remove_all_notices() {

		/** Work only on plugin settings page. */
		$screen = get_current_screen();
		if ( $screen->base != "toplevel_page_mdp_appender_settings" ) { return; }

		/** Remove other notices. */
		remove_all_actions( 'admin_notices' );
		remove_all_actions( 'all_admin_notices' );

	}

	/**
	 * Remove "Thank you for creating with WordPress" and WP version only from plugin settings page.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	public function remove_wp_copyrights() {

		/** Remove "Thank you for creating with WordPress" and WP version from plugin settings page. */
		$screen = get_current_screen(); // Get current screen.

		/** Appender Settings Page. */
		if ( $screen->base == 'toplevel_page_mdp_appender_settings' ) {
			add_filter( 'admin_footer_text', '__return_empty_string', 11 );
			add_filter( 'update_footer', '__return_empty_string', 11 );
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	private function public_hooks() {

		/** Load CSS and JavaScript for Frontend Area. */
		add_action( 'wp_enqueue_scripts', [ $this, 'styles' ] ); // JS.
		add_action( 'wp_enqueue_scripts', [ $this, 'scripts' ] ); // JS.

		/** JavaScript Required. */
		add_action( 'wp_footer', [$this, 'javascript_required'] );

	}

	/**
	 * Protect Content if JavaScript is Disabled.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function javascript_required() {

		if ( Settings::get_instance()->options['javascript'] != 'on' ) { return; }

		ob_start();
		?>
		<noscript>
			<div id='mdp-appender-js-disabled'>
				<div><?php echo wp_kses_post( Settings::get_instance()->options['javascript_msg'] ); ?></div>
			</div>
			<style>
				#mdp-appender-js-disabled {
					position: fixed;
					top: 0;
					left: 0;
					height: 100%;
					width: 100%;
					z-index: 999999;
					text-align: center;
					background-color: #FFFFFF;
					color: #000000;
					font-size: 40px;
					display: flex;
					align-items: center;
					justify-content: center;
				}
			</style>
		</noscript>
		<?php
		$result = ob_get_clean();

		echo $result;
	}

	/**
	 * Initialize main variables.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function initialization() {

		/** Plugin version. */
		if ( ! function_exists('get_plugin_data') ) {
			/** @noinspection PhpIncludeInspection */
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		$plugin_data = get_plugin_data( __FILE__ );
		self::$version = $plugin_data['Version'];

		/** Gets the plugin URL (with trailing slash). */
		self::$url = plugin_dir_url( __FILE__ );

		/** Gets the plugin PATH. */
		self::$path = plugin_dir_path( __FILE__ );

		/** Use minified libraries if SCRIPT_DEBUG is turned off. */
		self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		/** Set plugin basename. */
		self::$basename = plugin_basename( __FILE__ );

		/** Initialize plugin settings. */
		Settings::get_instance();

		/** Initialize PluginHelper. */
		PluginHelper::get_instance();

		/** Plugin update mechanism enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			PluginUpdater::get_instance();
		}

	}


	/**
	 * Register CSS for the public-facing side of the site.
	 *
	 * @return void
	 * @since 1.0.0
	 **/
	public function styles() {

		/** Checks if plugin should work on this page. */
		if ( ! AssignmentsTab::get_instance()->display() ) { return; }

		wp_enqueue_style( 'mdp-appender', self::$url . 'css/appender' . self::$suffix . '.css', [], self::$version );

		/** Add Custom CSS from settings tab. */
		wp_add_inline_style( 'mdp-appender', Settings::get_instance()->options['custom_css'] );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @return void
	 * @since 1.0.0
	 **/
	public function scripts() {

		/** Checks if plugin should work on this page. */
		if ( ! AssignmentsTab::get_instance()->display() ) { return; }

		wp_enqueue_script( 'moment' );
		wp_enqueue_script( 'mdp-appender', self::$url . 'js/appender' . self::$suffix . '.js', ['moment'], self::$version, true );
		wp_localize_script( 'mdp-appender', 'mdp_appender', [
			'stringLength'          => Settings::get_instance()->options['string_length'],
			'add_before'            => Settings::get_instance()->options['add_before'],
			'text_before'           => $this->replace_tags( Settings::get_instance()->options['text_before'] ),
			'add_after'             => Settings::get_instance()->options['add_after'],
			'text_after'            => $this->replace_tags( Settings::get_instance()->options['text_after'] ),
            'save'                  => Settings::get_instance()->options['save'],
            'view_source'           => Settings::get_instance()->options['view_source'],
            'developer_tool'        => Settings::get_instance()->options['developer_tool'],
            'safari_reader'         => Settings::get_instance()->options['safari_reader'],
            'js_required'           => Settings::get_instance()->options['js_required'],
            'js_required_message'   => Settings::get_instance()->options['js_required_message'],
			'wp_date_format'        => get_option( 'date_format' ),
			'wp_time_format'        => get_option( 'time_format' )
		] );

	}

	/**
	 * Replace tags like {SiteName} to correct values.
	 *
	 * @param $text string - The text in which replacements will be made.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return string|string[]
	 */
	private function replace_tags( $text ) {
		global $wp;

		/** Prepare variables. */
		$site_name = get_bloginfo( 'name' );
		$tagline = get_bloginfo( 'description' );
		$site_url = get_bloginfo( 'url' );

		if ( is_front_page() ) {
			$page_title = trim( get_the_title( get_option('page_on_front') ) );
		} else {
			$page_title = trim( wp_title( '', false ) );
		}

		$page_url = home_url( $wp->request );

		/** Do replacements. */
		$text = str_replace( '{SiteTitle}', $site_name, $text );
		$text = str_replace( '{Tagline}', $tagline, $text );
		$text = str_replace( '{SiteURL}', $site_url, $text );
		$text = str_replace( '{PageTitle}', $page_title, $text );
		$text = str_replace( '{PageURL}', $page_url, $text );

		//{Date} and {Time} replacement in JS.

		return $text;
	}

	/**
	 * Loads the Appender translated strings.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function load_textdomain() {

		load_plugin_textdomain( 'appender', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

	}

	/**
	 * Add CSS for admin area.
	 *
	 * @return void
	 * @since 1.0.0
	 **/
	public function admin_styles() {

		/** Get current screen to add styles on specific pages. */
		$screen = get_current_screen();

		/** Appender Settings Page. */
		if ( $screen->base == 'toplevel_page_mdp_appender_settings' ) {
			wp_enqueue_style( 'merkulov-ui', self::$url . 'css/merkulov-ui' . self::$suffix . '.css', array(), self::$version );
			wp_enqueue_style( 'mdp-appender-admin', self::$url . 'css/admin' . self::$suffix . '.css', [], self::$version );

			/** Appender popup on update. */
		} elseif ( 'plugin-install' == $screen->base ) {

			/** Styles only for our plugin. */
			if ( isset( $_GET['plugin'] ) AND $_GET['plugin'] === 'appender' ) {
				wp_enqueue_style( 'mdp-appender-plugin-install', self::$url . 'css/plugin-install' . self::$suffix . '.css', [], self::$version );
			}
		}

	}

	/**
	 * Add JS for admin area.
	 *
	 * @return void
	 * @since 1.0.0
	 **/
	public function admin_scripts() {

		/** Get current screen to add scripts on specific pages. */
		$screen = get_current_screen();

		if ( $screen->base == "toplevel_page_mdp_appender_settings" ) {
			wp_enqueue_script( 'merkulov-ui', self::$url . 'js/merkulov-ui' . self::$suffix . '.js', array(), self::$version, true );
			wp_enqueue_script( 'mdp-appender-admin', self::$url . 'js/admin' . self::$suffix . '.js', ['jquery', 'jquery-ui-accordion', 'jquery-ui-sortable'], self::$version, true );
		}

	}

	/**
	 * Run when the plugin is activated.
	 *
	 * @static
	 * @since 1.0.0
	 **/
	public static function on_activation() {

		/** Security checks. */
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}

		$plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
		check_admin_referer( "activate-plugin_{$plugin}" );

		/** Send install Action to our host. */
		Helper::get_instance()->send_action( 'install', 'appender', self::$version );

	}

	/**
	 * Main Appender Instance.
	 *
	 * Insures that only one instance of Appender exists in memory at any one time.
	 *
	 * @static
	 * @return Appender
	 * @since 1.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Appender ) ) {
			self::$instance = new Appender;
		}

		return self::$instance;
	}

} // End Class Appender.

/** Run when the plugin is activated. */
register_activation_hook( __FILE__, ['Merkulove\Appender', 'on_activation'] );

/** Run Appender class. */
Appender::get_instance();