/**
 * Appender - Adds copyrights to the copied text.
 * Exclusively on Envato Market: https://1.envato.market/appender
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       Copyright (C) 2018 - 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         dmitry@merkulov.design
 **/

"use strict";

const MdpAppender = ( function () {

    "use strict";

    function _appender() {

        function init() {

            /** Disable Save. */
            if ( mdp_appender.save == 'on' ) { disable_save(); }

            /** Disable View Source. */
            if ( mdp_appender.view_source == 'on' ) { disable_view_source(); }

            /** Disable Developer Tool. */
            if ( mdp_appender.developer_tool == 'on' ) { disable_developer_tool(); }

            /** Disable Reader Mode. */
            if ( mdp_appender.safari_reader == 'on' ) { disable_reader_mode(); }

            /** Add custom content to Clipboard. */
            document.addEventListener('copy', addToClipboard );

        }

        /**
         * Add custom content to Clipboard.
         **/
        function addToClipboard( e ) {
            e.preventDefault();

            let beforeText = '';
            let afterText = '';
            let stringLength = parseInt( mdp_appender.stringLength, 10 );
            let selectedText = window.getSelection().toString();

            if ( 'on' === mdp_appender.add_before ) {
                beforeText = mdp_appender.text_before + '\n';
                beforeText = dataReplacements( beforeText );
                beforeText = timeReplacements( beforeText );
            }

            if ( 'on' === mdp_appender.add_after ) {
                afterText = '\n' + mdp_appender.text_after;
                afterText = dataReplacements( afterText );
                afterText = timeReplacements( afterText );
            }

            if ( stringLength !== 0 && selectedText.length > stringLength ) {
                selectedText = selectedText.substring( 0, stringLength );
            }

            let copiedText =  beforeText + selectedText + afterText;
            let clipData = e.clipboardData || window.clipboardData;
            if ( clipData ) {
                 clipData.setData( 'Text', copiedText );
            }
        }

        /**
         * Replace tag {Date} to correct values.
         **/
        function dataReplacements( text ) {

            let wp_date_format = mdp_appender.wp_date_format;
            let formatted_date = moment().format( phpToMoment( wp_date_format ) );

            return text.replace( /{Date}/g, formatted_date );
        }

        /**
         * Replace tag {Time} to correct values.
         **/
        function timeReplacements( text ) {

            let wp_time_format = mdp_appender.wp_time_format;
            let formatted_time = moment().format( phpToMoment( wp_time_format ) );

            return text.replace( /{Time}/g, formatted_time );
        }

        /**
         * Convert PHP date string to Moment.js format.
         **/
        function phpToMoment(str) {

            let replacements = {
                'd' : 'DD',
                'D' : 'ddd',
                'j' : 'D',
                'l' : 'dddd',
                'N' : 'E',
                'S' : 'o',
                'w' : 'e',
                'z' : 'DDD',
                'W' : 'W',
                'F' : 'MMMM',
                'm' : 'MM',
                'M' : 'MMM',
                'n' : 'M',
                't' : '', // no equivalent
                'L' : '', // no equivalent
                'o' : 'YYYY',
                'Y' : 'YYYY',
                'y' : 'YY',
                'a' : 'a',
                'A' : 'A',
                'B' : '', // no equivalent
                'g' : 'h',
                'G' : 'H',
                'h' : 'hh',
                'H' : 'HH',
                'i' : 'mm',
                's' : 'ss',
                'u' : 'SSS',
                'e' : 'zz', // deprecated since Moment.js 1.6.0
                'I' : '', // no equivalent
                'O' : '', // no equivalent
                'P' : '', // no equivalent
                'T' : '', // no equivalent
                'Z' : '', // no equivalent
                'c' : '', // no equivalent
                'r' : '', // no equivalent
                'U' : 'X'
            };

            return str.split('').map(chr => chr in replacements ? replacements[chr] : chr).join('');
        }


        /**
         * Disable Save, HotKeys: Ctrl+S, ⌘+S.
         * Protect Your Text from Being Saved by Save HotKeys.
         **/
        function disable_save() {

            disable_key( 83 ); // Ctrl+S, ⌘+S.

        }

        /**
         * Disable View Source, HotKeys: Ctrl+U, ⌘+U.
         * Disable to View Source Code of Page by HotKeys.
         **/
        function disable_view_source() {

            disable_key( 85 ); // Ctrl+U, ⌘+U.

        }

        /**
         * Disable Reader Mode in Safari, HotKeys: ⌘+Shift+P.
         * Protect Your Page from Being open in Reader mode
         **/
        function disable_reader_mode() {

            if ( navigator.userAgent.toLowerCase().includes( 'safari' ) && !navigator.userAgent.toLowerCase().includes( 'chrome' ) ) {

                window.addEventListener( 'keydown', function( e ) {

                    if ( ( e.ctrlKey || e.metaKey ) && e.shiftKey && e.keyCode === 82 ) {
                        e.preventDefault();
                    }

                } );

            }

        }

        /**
         * Disable Developer Tool, HotKeys: Ctrl+Shift+I, ⌘+⌥+I, F12
         * Disable to View Source Code of Page by Developer Tools.
         **/
        function disable_developer_tool() {

            window.addEventListener( 'keydown', function( e ) {

                if (
                    e.keyCode === 123 || // F12
                    ( ( e.ctrlKey || e.metaKey ) && e.shiftKey && e.keyCode === 73 ) // Ctrl+Shift+I, ⌘+⌥+I
                ) {
                    e.preventDefault();
                }

            } );

        }

        /**
         * Disable CTRL|CMD + Key by key Code.
         *
         * @param {number} code
         **/
        function disable_key( code ) {

            window.addEventListener( 'keydown', function( e ) {

                /** For Windows Check CTRL. */
                if ( e.ctrlKey && e.which === code ) { e.preventDefault(); }

                /** For Mac Check Metakey. */
                if ( e.metaKey && e.which === code ) { e.preventDefault(); }

            } );

            document.keypress = function( e ) {

                /** For Windows Check CTRL. */
                if ( e.ctrlKey && e.which === code ) { return false; }

                /** For Mac Check Metakey. */
                if ( e.metaKey && e.which === code ) { return false; }

            };

        }

        return {
            init: init
        };

    }

    return _appender;

} )();

document.addEventListener( 'DOMContentLoaded', function () {

    let mdpAppender = new MdpAppender();
    mdpAppender.init();

});